import os
import sys
import time
from quality_sort import call_sortseqs_from_func
from joinseqs import call_joinseqs_from_func
from SSR_search import call_findSSRs_from_func


def readparams(f):

    analysis_params = ''
    while True:
        input_line = f.readline().strip()
        if input_line == ';':
            break
        else:
            analysis_params += (input_line + '\n')
    return analysis_params

def main():
    
    try:
        paramfile = sys.argv[1]
    except:
        sys.exit("""
Usage: 
    SSR_pipeline.py name_of_configuration_file
See documentation for examples of configuration files.""")

    try:
        f = open(paramfile, 'r')
    except:
        sys.exit("Error. Check filename passed to program.")         
        
    firstline = f.readline()
    #variable 'nSets' is set by executing the first line of the config file
    exec(firstline.strip()) #nSets is assigned here

    for aa in range(nSets):
        #dynamically evaluate strings from the configuration file to set
        #variables assoicated with the analysis:
        params = readparams(f)
            #read_set_1 and read_set_2: lists of files containing 
                #paired-end reads
            #path_to_files: specifies the location of the files to be analyzed.
                #Please prefix this name with an 'r' on windows machines to
                #ensure proper function (backslashes have different meaning)
            #generic_output_name: a generic name associated with these files.
                #may be a speciesname, barcode nam, or any other desired label
            #keepZipped: either 'T' (true) or 'F' (false). If true, then 
                #intermediate files are gzipped to save disk space at the
                #expense of execution speed. Note that the final files
                #containing SSRs are never zipped
            #alignment_Parameters: a list containing three parameters from
                #the FLASH alignment algorithm. Present in the form
                # [min_overlap, mismatch_ratio_threshold, max_overlap]
            #microsat_search_params: a list of the sets of analysis parameters
                #for the SSR search. Each search is in the form
                #(repeat_len, min_number_of_repeats, flanking_sequence_len).
                #Multiple sets of search parameters are permitted.
        exec(params)
            
        nFilePairs = len(read_set_1)
        os.chdir(path_to_files)
        try:
            os.mkdir(generic_output_name)
        except:
            pass
        
        ##code for quality sort
        print ''
        print 'Starting set ' + str(aa+1)
        print "Calling 'quality_sort.py': " + time.asctime()                
        for bb in range(nFilePairs):
            file1name = read_set_1[bb]
            file2name = read_set_2[bb]
            
            call_sortseqs_from_func(file1name, file2name, generic_output_name,
                                    keepZipped)

        #code for paired alignment        
        print "Calling 'joinseqs.py': " + time.asctime()
        os.chdir(generic_output_name)
        for bb in range(nFilePairs):
            if keepZipped == 'T':
                f1name = read_set_1[bb][:-9] + '_quality_passed.fastq.gz'
                f2name = read_set_2[bb][:-9] + '_quality_passed.fastq.gz'
            else:
                f1name = read_set_1[bb][:-6] + '_quality_passed.fastq'
                f2name = read_set_2[bb][:-6] + '_quality_passed.fastq'
            min_overlap = alignment_Parameters[0]
            match_ratio_threshold = alignment_Parameters[1]
            max_overlap = alignment_Parameters[2]
            
            call_joinseqs_from_func(f1name, f2name, min_overlap, 
                                    match_ratio_threshold, max_overlap, 
                                    keepZipped)

        #code for SSR searches
        print "Calling 'SSR_search.py': " + time.asctime()
        for paramset in microsat_search_params:
            repeat_len = paramset[0]
            min_repeats = paramset[1]
            flank_len = paramset[2]
            #outfile_name = generic_output_name + '_SSRs
            for bb in range(nFilePairs):
                if keepZipped == 'T':
                    filename = ('joined_reads_' + read_set_1[bb][:-9] + 
                                '_quality_passed.fastq.gz')
                else:
                    filename = ('joined_reads_' + read_set_1[bb][:-6] + 
                                '_quality_passed.fastq')
                                
                if bb == 0:
                    startnewfile = True
                else:
                    startnewfile = False

                call_findSSRs_from_func(filename, repeat_len, 
                                        min_repeats, flank_len, startnewfile)
    
        print 'set ' + str(aa + 1) + ' complete :' + time.asctime()
        print ''

        os.chdir('..')

if __name__ == '__main__':
    main()









