#cython: boundscheck=False
#cython: wraparound=False

class matchinfo:
    def __init__(self):
        self.matchposition = 0
        self.matchlen = 0
        self.bestratio = 1.0
        self.overlap_seq = 'X'
        self.overlap_qual = 'X'
        
def lib_version():
    print "Cython"
        
def joinseqs(char seq1[], char seq2[], char qual1[], char qual2[], 
             int minoverlap, double cutoff_ratio, int max_overlap, bestmatch):

    cdef int lens1, lens2, maxlen, lendiff, aa, bb, nnucs, matchpos
    cdef int counter, cases, int_number, matchindex, n_best_matches
    cdef double bestratio, current_ratio, qualscore, bestqual
    cdef bytes overlap_seq, overlap_qual
    cdef bytes n1, n2, q1, q2
    cdef char n1c, n2c
    cdef list best_match_positions

    maxlen = len(seq1)

    #step 1: find the alignment overlap that minimizes "current_ratio"
    bestratio = 1.0
    best_match_positions = [0]
    n_best_matches = 0
    for aa in range(minoverlap, maxlen + 1):
        current_ratio = 0.0
        nnucs = 0
        for bb in range(aa):
            n1c = seq1[maxlen-aa+bb]
            n2c = seq2[bb]
            if (n1c != 'N') and (n2c != 'N'):
                nnucs = nnucs + 1
                if (n1c != n2c):
                    current_ratio = current_ratio + 1
        if nnucs > 0:
            if nnucs > max_overlap:
                current_ratio = current_ratio/max_overlap
            else:
                current_ratio = current_ratio/nnucs
            if current_ratio < bestratio:
                bestratio = current_ratio
                best_match_positions = [aa]
                n_best_matches = 1
            elif current_ratio == bestratio:
                best_match_positions.append(aa)

        
    if (bestratio <= cutoff_ratio):
        #consider the possibility of "tied" alignments (if they occur) and
        #pick the alignment with the highest average quality score at 
        #ambiguous sites
        bestqual = -1
        counter = -1
        if n_best_matches > 1: #do this only if there are ties
            for matchpos in best_match_positions:
                counter = counter + 1
    
                qualscore = 0
                cases = 0
                int_number = maxlen - matchpos
                for aa in range(matchpos):
                    n1c = seq1[int_number + aa]
                    n2c = seq2[aa]
                    if n1c != 'N' and n2c != 'N':
                        if n1c != n2c:
                            cases = cases + 1
                            qualscore = qualscore + qual1[int_number + aa]
                            qualscore = qualscore + qual2[aa]
                if cases > 0:
                    qualscore /= (2*cases)
                    if qualscore >= bestqual:
                        bestqual = qualscore
                        matchindex = best_match_positions[counter]
                else:
                    matchindex = best_match_positions[counter]
        else: #if there are no ties, many calculations can be skipped
            matchindex = best_match_positions[0]

        #take disagreements between sequences into consideration.
        #Nucleotide w/ highest quality score gets used in alignment
        #when disagreements occur.
        overlap_seq = ''.encode()
        overlap_qual = ''.encode()
        int_number = maxlen - matchindex
        for aa in range(matchindex):
            n1 = seq1[int_number + aa]
            n2 = seq2[aa]
            q1 = qual1[int_number + aa]
            q2 = qual2[aa]
            if n1 != 'N' and n2 != 'N':
                if n1 == n2:
                    overlap_seq = overlap_seq + n1
                    overlap_qual = overlap_qual + max(q1,q2)
                elif n1 != n2:
                    overlap_qual = overlap_qual + '#'
                    if q1 > q2:
                        overlap_seq = overlap_seq + n1
                    else:
                        overlap_seq = overlap_seq + n2
            elif n1 == 'N' and n2 == 'N':
                overlap_seq = overlap_seq + 'N'
                overlap_qual = overlap_qual + max(q1,q2)#'#'
            elif n1 == 'N' and n2 != 'N':
                overlap_seq = overlap_seq + n2
                overlap_qual = overlap_qual + q2
            elif n1 != 'N' and n2 == 'N':
                overlap_seq = overlap_seq + n1
                overlap_qual = overlap_qual + q1
                
        bestmatch.matchlen = len(overlap_seq)
        bestmatch.bestratio = bestratio
        bestmatch.overlap_seq = overlap_seq
        bestmatch.overlap_qual = overlap_qual
        bestmatch.matchposition = maxlen - bestmatch.matchlen


    
