import gzip
import sys
import os
import time

def sort_seqs(file1name, file2name, output_subdir_name, keepZipped):
 
    time1 = time.asctime() 
    n_pairs_checked = 0
    n_pairs_passed = 0
    #assume that files either have the file extensions ".fastq.gz" or ".fastq"
    if file1name[-3:] == '.gz':
        file1basename = file1name[:-9]
        file2basename = file2name[:-9]
        infile1 = gzip.open(file1name, 'r')
        infile2 = gzip.open(file2name, 'r')
    else:
        file1basename = file1name[:-6]
        file2basename = file2name[:-6]
        infile1 = open(file1name, 'r')
        infile2 = open(file2name, 'r')
        
    if keepZipped:
        outname1 = os.path.join(output_subdir_name, file1basename + 
                                '_quality_passed.fastq.gz')
        outname2 = os.path.join(output_subdir_name, file2basename + 
                                '_quality_passed.fastq.gz')
        outfile1 = gzip.open(outname1, 'w')
        outfile2 = gzip.open(outname2, 'w')
    else:
        outname1 = os.path.join(output_subdir_name, file1basename + 
                                '_quality_passed.fastq')
        outname2 = os.path.join(output_subdir_name, file2basename + 
                                '_quality_passed.fastq')
        outfile1 = open(outname1, 'w')
        outfile2 = open(outname2, 'w')
    
    for line in infile1:
        line1_1 = line
        line1_2 = infile1.next()
        line1_3 = infile1.next()
        line1_4 = infile1.next()
        
        line2_1 = infile2.next()
        line2_2 = infile2.next()
        line2_3 = infile2.next()
        line2_4 = infile2.next()
        
        passedinfo_1 = line1_1.strip().split(':')[7]
        passedinfo_2 = line2_1.strip().split(':')[7]

        n_pairs_checked += 1
    
        if passedinfo_1 == 'N' and passedinfo_2 == 'N':
            n_pairs_passed += 1
            outfile1.write(line1_1)
            outfile1.write(line1_2)
            outfile1.write(line1_3)
            outfile1.write(line1_4)
    
            outfile2.write(line2_1)
            outfile2.write(line2_2)
            outfile2.write(line2_3)
            outfile2.write(line2_4)
    
    infile1.close()
    infile2.close()
    outfile1.close()
    outfile2.close()    
    f = open(os.path.join(output_subdir_name,'analysis_report.txt'), 'a')
    f.write('\n\n**Module quality_sort**:\n')
    f.write("Analysis start: " + time1 + '\n')    
    reportstr = "Quality sort: \n" + str(n_pairs_checked) 
    reportstr += (' reads examined.\n' + str(n_pairs_passed) + 
                ' reads retained.\n')
    f.write(reportstr)
    f.write("Analysis complete: " + time.asctime() + '\n')    
    f.close()

def call_sortseqs_from_cl():
    try:
        file1 = sys.argv[1]
        file2 = sys.argv[2]
        output_subdir_name = sys.argv[3]
        keepZipped = sys.argv[4]
    except:
        sys.exit("""
Error. Check filenames and arguments passed to program.
Program usage is as follows:
    quality_sort.py f1 f2 subdir keepZipped
where
    f1 = complete name and path to file 1
    f2 = complete name and path to file 2
    subdir = name of a subdirectory to f1 and f2 where output will be written
    keepZipped = T or F, corresponding to True or False. If True, then output
        files will be written as gzipped archives to save disk space.
""")   
    
    if keepZipped == 'T':
        keepZipped = True
    else:
        keepZipped = False
        
    file1name = os.path.basename(file1)
    file2name = os.path.basename(file2)
    path_to_files = os.path.dirname(file1)

    os.chdir(path_to_files)
    try:
        os.mkdir(output_subdir_name)
    except:
        pass

    sort_seqs(file1name, file2name, output_subdir_name, keepZipped)

    os.chdir('..')


def call_sortseqs_from_func(file1name, file2name, output_subdir_name, 
                            keepZipped):
    if keepZipped == 'T':
        keepZipped = True
    else:
        keepZipped = False

    sort_seqs(file1name, file2name, output_subdir_name, keepZipped)

if __name__ == '__main__':
    call_sortseqs_from_cl()    